@@ -14,6 +14,7 @@
 #    under the License.
 
 import gzip
+import io
 import logging
 import math
 import os
@@ -31,7 +32,6 @@ from oslo_utils import excutils
 from oslo_utils import imageutils
 from oslo_utils import units
 import requests
-import six
 
 from ironic_lib.common.i18n import _
 from ironic_lib import disk_partitioner
@@ -51,10 +51,10 @@ opts = [
     cfg.StrOpt('dd_block_size',
                default='1M',
                help='Block size to use when writing to the nodes disk.'),
-    cfg.IntOpt('iscsi_verify_attempts',
+    cfg.IntOpt('partition_detection_attempts',
                default=3,
-               help='Maximum attempts to verify an iSCSI connection is '
-                    'active, sleeping 1 second between attempts.'),
+               min=1,
+               help='Maximum attempts to detect a newly created partition.'),
     cfg.IntOpt('partprobe_attempts',
                default=10,
                help='Maximum number of attempts to try to read the '
@@ -67,7 +67,7 @@ CONF.register_opts(opts, group='disk_utils')
 LOG = logging.getLogger(__name__)
 
 _PARTED_PRINT_RE = re.compile(r"^(\d+):([\d\.]+)MiB:"
-                              "([\d\.]+)MiB:([\d\.]+)MiB:(\w*)::(\w*)")
+                              "([\d\.]+)MiB:([\d\.]+)MiB:(\w*):(.*):(.*);")
 
 CONFIGDRIVE_LABEL = "config-2"
 MAX_CONFIG_DRIVE_SIZE_MB = 64
@@ -84,7 +84,8 @@ def list_partitions(device):
 
     :param device: The device path.
     :returns: list of dictionaries (one per partition) with keys:
-              number, start, end, size (in MiB), filesystem, flags
+              number, start, end, size (in MiB), filesystem, partition_name,
+              flags
     """
     output = utils.execute(
         'parted', '-s', '-m', device, 'unit', 'MiB', 'print',
@@ -93,7 +94,8 @@ def list_partitions(device):
         output = output.decode("utf-8")
     lines = [line for line in output.split('\n') if line.strip()][2:]
     # Example of line: 1:1.00MiB:501MiB:500MiB:ext4::boot
-    fields = ('number', 'start', 'end', 'size', 'filesystem', 'flags')
+    fields = ('number', 'start', 'end', 'size', 'filesystem', 'partition_name',
+              'flags')
     result = []
     for line in lines:
         match = _PARTED_PRINT_RE.match(line)
@@ -157,6 +159,41 @@ def get_disk_identifier(dev):
     return disk_identifier[0]
 
 
+def get_uefi_disk_identifier(dev):
+    """Get the uuid from the disk being exposed by the ramdisk.
+
+    This uuid is appended to the pxe config which will then be set as the root
+    and load the bootx64.efi file using chainloader and boot the machine.
+    This is helpful in deployments to nodes with multiple disks.
+
+    https://wiki.gentoo.org/wiki/GRUB2/Chainloading
+
+    :param dev: Path for the already populated disk device.
+    :raises InstanceDeployFailure: Image is not UEFI bootable.
+    :returns: The UUID of the partition.
+    """
+    partition_id = None
+    try:
+        report, _ = utils.execute('fdisk', '-l', dev, run_as_root=True)
+    except processutils.ProcessExecutionError as e:
+        msg = _('Failed to find the partition on the disk %s ') % e
+        LOG.error(msg)
+        raise exception.InstanceDeployFailure(msg)
+    for line in report.splitlines():
+        if line.startswith(dev) and 'EFI System' in line:
+            vals = line.split()
+            partition_id = vals[0]
+    try:
+        lsblk_output, _ = utils.execute('lsblk', '-PbioUUID', partition_id,
+                                        run_as_root=True)
+        disk_identifier = lsblk_output.split("=")[1].strip()
+        disk_identifier = disk_identifier.strip('"')
+    except processutils.ProcessExecutionError as e:
+        raise exception.InstanceDeployFailure("Image is not UEFI bootable. "
+                                              "Error: %s " % e)
+    return disk_identifier
+
+
 def is_iscsi_device(dev, node_uuid):
     """check whether the device path belongs to an iscsi device. """
 
@@ -164,9 +201,11 @@ def is_iscsi_device(dev, node_uuid):
     return iscsi_id in dev
 
 
-def is_nvme_device(dev):
-    """check whether the device path belongs to an NVMe drive. """
-    return "/dev/nvme" in dev
+def is_last_char_digit(dev):
+    """check whether device name ends with a digit"""
+    if len(dev) >= 1:
+        return dev[-1].isdigit()
+    return False
 
 
 def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
@@ -214,7 +253,7 @@ def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
     # the device partitions as /dev/sda1 and not /dev/sda-part1.
     if is_iscsi_device(dev, node_uuid):
         part_template = dev + '-part%d'
-    elif is_nvme_device(dev):
+    elif is_last_char_digit(dev):
         part_template = dev + 'p%d'
     else:
         part_template = dev + '%d'
@@ -290,12 +329,13 @@ def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
     if commit:
         # write to the disk
         dp.commit()
+        _trigger_device_rescan(dev)
     return part_dict
 
 
 def is_block_device(dev):
     """Check whether a device is block or not."""
-    attempts = CONF.disk_utils.iscsi_verify_attempts
+    attempts = CONF.disk_utils.partition_detection_attempts
     for attempt in range(attempts):
         try:
             s = os.stat(dev)
@@ -313,10 +353,15 @@ def is_block_device(dev):
     raise exception.InstanceDeployFailure(msg)
 
 
-def dd(src, dst):
+def dd(src, dst, conv_flags=None):
     """Execute dd from src to dst."""
+    if conv_flags:
+        extra_args = ['conv=%s' % conv_flags]
+    else:
+        extra_args = []
+
     utils.dd(src, dst, 'bs=%s' % CONF.disk_utils.dd_block_size, 'oflag=direct',
-             'conv=sparse')
+             *extra_args)
 
 
 def qemu_img_info(path):
@@ -336,10 +381,10 @@ def convert_image(source, dest, out_format, run_as_root=False):
     utils.execute(*cmd, run_as_root=run_as_root, prlimit=QEMU_IMG_LIMITS)
 
 
-def populate_image(src, dst):
+def populate_image(src, dst, conv_flags=None):
     data = qemu_img_info(src)
     if data.file_format == 'raw':
-        dd(src, dst)
+        dd(src, dst, conv_flags=conv_flags)
     else:
         convert_image(src, dst, 'raw', True)
 
@@ -407,7 +452,20 @@ def destroy_disk_metadata(dev, node_uuid):
                 utils.execute('wipefs', '--all', dev,
                               run_as_root=True,
                               use_standard_locale=True)
-
+    # NOTE(TheJulia): sgdisk attempts to load and make sense of the
+    # partition tables in advance of wiping the partition data.
+    # This means when a CRC error is found, sgdisk fails before
+    # erasing partition data.
+    # This is the same bug as
+    # https://bugs.launchpad.net/ironic-python-agent/+bug/1737556
+    dd_device = 'of=%s' % dev
+    gpt_backup = get_dev_block_size(dev) - 33
+    dd_seek = 'seek=%i' % gpt_backup
+    utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, 'count=33',
+                  run_as_root=True, use_standard_locale=True)
+    utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, 'count=33',
+                  dd_seek, run_as_root=True, use_standard_locale=True)
+    # Go ahead and let sgdisk run as well.
     utils.execute('sgdisk', '-Z', dev, run_as_root=True,
                   use_standard_locale=True)
 
@@ -450,7 +508,7 @@ def _get_configdrive(configdrive, node_uuid, tempdir=None):
         data = configdrive
 
     try:
-        data = six.BytesIO(base64.decode_as_bytes(data))
+        data = io.BytesIO(base64.decode_as_bytes(data))
     except TypeError:
         error_msg = (_('Config drive for node %s is not base64 encoded '
                        'or the content is malformed.') % node_uuid)
@@ -486,7 +544,7 @@ def _get_configdrive(configdrive, node_uuid, tempdir=None):
 def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
                  image_path, node_uuid, preserve_ephemeral=False,
                  configdrive=None, boot_option="netboot", boot_mode="bios",
-                 tempdir=None, disk_label=None, cpu_arch=""):
+                 tempdir=None, disk_label=None, cpu_arch="", conv_flags=None):
     """Create partitions and copy an image to the root partition.
 
     :param dev: Path for the device to work on.
@@ -496,7 +554,8 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
         no ephemeral partition will be created.
     :param ephemeral_format: The type of file system to format the ephemeral
         partition.
-    :param image_path: Path for the instance's disk image.
+    :param image_path: Path for the instance's disk image. If ``None``,
+        the root partition is prepared but not populated.
     :param node_uuid: node's uuid. Used for logging.
     :param preserve_ephemeral: If True, no filesystem is written to the
         ephemeral block device, preserving whatever content it had (if the
@@ -514,10 +573,14 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
         steps will be taken. This default should be used for x86_64. When
         set to ppc64*, architecture specific steps are taken for booting a
         partition image locally.
+    :param conv_flags: Flags that need to be sent to the dd command, to control
+        the conversion of the original file when copying to the host. It can
+        contain several options separated by commas.
     :returns: a dictionary containing the following keys:
         'root uuid': UUID of root partition
         'efi system partition uuid': UUID of the uefi system partition
         (if boot mode is uefi).
+        `partitions`: mapping of partition types to their device paths.
         NOTE: If key exists but value is None, it means partition doesn't
         exist.
     """
@@ -576,7 +639,7 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
 
         if configdrive_part:
             # Copy the configdrive content to the configdrive partition
-            dd(configdrive_file, configdrive_part)
+            dd(configdrive_file, configdrive_part, conv_flags=conv_flags)
             LOG.info("Configdrive for node %(node)s successfully copied "
                      "onto partition %(partition)s",
                      {'node': node_uuid, 'partition': configdrive_part})
@@ -587,9 +650,13 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
         if configdrive_file:
             utils.unlink_without_raise(configdrive_file)
 
-    populate_image(image_path, root_part)
-    LOG.info("Image for %(node)s successfully populated",
-             {'node': node_uuid})
+    if image_path is not None:
+        populate_image(image_path, root_part, conv_flags=conv_flags)
+        LOG.info("Image for %(node)s successfully populated",
+                 {'node': node_uuid})
+    else:
+        LOG.debug("Root partition for %s was created, but not populated",
+                  node_uuid)
 
     if swap_part:
         utils.mkfs(fs='swap', path=swap_part, label='swap1')
@@ -623,7 +690,7 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
         with excutils.save_and_reraise_exception():
             LOG.error("Failed to detect %s", part)
 
-    return uuids_to_return
+    return dict(partitions=part_dict, **uuids_to_return)
 
 
 def list_opts():
@@ -693,7 +760,7 @@ def _get_labelled_partition(device_path, label, node_uuid):
                    for v in shlex.split(device))}
             if not dev:
                 continue
-            if dev['LABEL'] == label:
+            if dev['LABEL'].upper() == label.upper():
                 if found_part:
                     found_2 = '/dev/%(part)s' % {'part': dev['NAME'].strip()}
                     found = [found_part, found_2]
@@ -742,12 +809,10 @@ def _fix_gpt_structs(device, node_uuid):
         commands fail.
     """
     try:
-        output, err = utils.execute('partprobe', device,
-                                    use_standard_locale=True,
-                                    run_as_root=True)
+        output, _err = utils.execute('sgdisk', '-v', device, run_as_root=True)
 
-        search_str = "fix the GPT to use all of the space"
-        if search_str in err:
+        search_str = "it doesn't reside\nat the end of the disk"
+        if search_str in output:
             utils.execute('sgdisk', '-e', device, run_as_root=True)
     except (processutils.UnknownArgumentError,
             processutils.ProcessExecutionError, OSError) as e:
@@ -780,6 +845,45 @@ def fix_gpt_partition(device, node_uuid):
         raise exception.InstanceDeployFailure(msg)
 
 
+def _trigger_device_rescan(device):
+    """Sync and trigger device rescan.
+
+    Disk partition performed via parted, when performed on a ramdisk
+    do not have to honor the fsync mechanism. In essence, fsync is used
+    on the file representing the block device, which falls to the kernel
+    filesystem layer to trigger a sync event. On a ramdisk using ramfs,
+    this is an explicit non-operation.
+
+    As a result of this, we need to trigger a system wide sync operation
+    which will trigger cache to flush to disk, after which partition changes
+    should be visible upon re-scan.
+
+    When ramdisks are not in use, this also helps ensure that data has
+    been safely flushed across the wire, such as on iscsi connections.
+
+    :param device: The block device containing paritions that is attempting
+                   to be updated.
+    """
+    # TODO(TheJulia): This helper method was broken out for re-use, and
+    # does not have an explicit test case for itself, and it should, but
+    # it's steps are fairly explicitly checked with mocks on execute in
+    # the partition code.
+    LOG.debug('Explicitly calling sync to force buffer/cache flush.')
+    utils.execute('sync')
+    # Make sure any additions to the partitioning are reflected in the
+    # kernel.
+    LOG.debug('Waiting until udev event queue is empty')
+    utils.execute('udevadm', 'settle')
+    try:
+        utils.execute('partprobe', device, run_as_root=True,
+                      attempts=CONF.disk_utils.partprobe_attempts)
+        # Also verify that the partitioning is correct now.
+        utils.execute('sgdisk', '-v', device, run_as_root=True)
+    except processutils.ProcessExecutionError as exc:
+        LOG.warning('Failed to verify partitioning after creating '
+                    'partition(s): %s', exc)
+
+
 def create_config_drive_partition(node_uuid, device, configdrive):
     """Create a partition for config drive
 
@@ -862,6 +966,8 @@ def create_config_drive_partition(node_uuid, device, configdrive):
                 utils.execute('parted', '-a', 'optimal', '-s', '--', device,
                               'mkpart', 'primary', 'fat32', startlimit,
                               endlimit, run_as_root=True)
+            # Trigger device rescan
+            _trigger_device_rescan(device)
 
             upd_parts = set(part['number'] for part in list_partitions(device))
             new_part = set(upd_parts) - set(cur_parts)
@@ -873,7 +979,7 @@ def create_config_drive_partition(node_uuid, device, configdrive):
 
             if is_iscsi_device(device, node_uuid):
                 config_drive_part = '%s-part%s' % (device, new_part.pop())
-            elif is_nvme_device(device):
+            elif is_last_char_digit(device):
                 config_drive_part = '%sp%s' % (device, new_part.pop())
             else:
                 config_drive_part = '%s%s' % (device, new_part.pop())
