@@ -13,7 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import eventlet
 import mock
 from testtools.matchers import HasLength
 
@@ -23,6 +22,9 @@ from ironic_lib.tests import base
 from ironic_lib import utils
 
 
+CONF = disk_partitioner.CONF
+
+
 class DiskPartitionerTestCase(base.IronicLibTestCase):
 
     def test_add_partition(self):
@@ -32,18 +34,22 @@ class DiskPartitionerTestCase(base.IronicLibTestCase):
         dp.add_partition(2048, boot_flag='boot')
         dp.add_partition(2048, boot_flag='bios_grub')
         expected = [(1, {'boot_flag': None,
+                         'extra_flags': None,
                          'fs_type': '',
                          'type': 'primary',
                          'size': 1024}),
                     (2, {'boot_flag': None,
+                         'extra_flags': None,
                          'fs_type': 'linux-swap',
                          'type': 'primary',
                          'size': 512}),
                     (3, {'boot_flag': 'boot',
+                         'extra_flags': None,
                          'fs_type': '',
                          'type': 'primary',
                          'size': 2048}),
                     (4, {'boot_flag': 'bios_grub',
+                         'extra_flags': None,
                          'fs_type': '',
                          'type': 'primary',
                          'size': 2048})]
@@ -57,14 +63,22 @@ class DiskPartitionerTestCase(base.IronicLibTestCase):
     def test_commit(self, mock_utils_exc, mock_disk_partitioner_exec):
         dp = disk_partitioner.DiskPartitioner('/dev/fake')
         fake_parts = [(1, {'boot_flag': None,
+                           'extra_flags': None,
                            'fs_type': 'fake-fs-type',
                            'type': 'fake-type',
                            'size': 1}),
                       (2, {'boot_flag': 'boot',
+                           'extra_flags': None,
                            'fs_type': 'fake-fs-type',
                            'type': 'fake-type',
                            'size': 1}),
                       (3, {'boot_flag': 'bios_grub',
+                           'extra_flags': None,
+                           'fs_type': 'fake-fs-type',
+                           'type': 'fake-type',
+                           'size': 1}),
+                      (4, {'boot_flag': 'boot',
+                           'extra_flags': ['prep', 'fake-flag'],
                            'fs_type': 'fake-fs-type',
                            'type': 'fake-type',
                            'size': 1})]
@@ -79,23 +93,28 @@ class DiskPartitionerTestCase(base.IronicLibTestCase):
             'mkpart', 'fake-type', 'fake-fs-type', '2', '3',
             'set', '2', 'boot', 'on',
             'mkpart', 'fake-type', 'fake-fs-type', '3', '4',
-            'set', '3', 'bios_grub', 'on')
+            'set', '3', 'bios_grub', 'on',
+            'mkpart', 'fake-type', 'fake-fs-type', '4', '5',
+            'set', '4', 'boot', 'on', 'set', '4', 'prep', 'on',
+            'set', '4', 'fake-flag', 'on')
         mock_utils_exc.assert_called_once_with(
             'fuser', '/dev/fake', run_as_root=True,
             check_exit_code=[0, 1])
 
-    @mock.patch.object(eventlet.greenthread, 'sleep', lambda seconds: None)
     @mock.patch.object(disk_partitioner.DiskPartitioner, '_exec',
                        autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
     def test_commit_with_device_is_busy_once(self, mock_utils_exc,
                                              mock_disk_partitioner_exec):
+        CONF.set_override('check_device_interval', 0, group='disk_partitioner')
         dp = disk_partitioner.DiskPartitioner('/dev/fake')
         fake_parts = [(1, {'boot_flag': None,
+                           'extra_flags': None,
                            'fs_type': 'fake-fs-type',
                            'type': 'fake-type',
                            'size': 1}),
                       (2, {'boot_flag': 'boot',
+                           'extra_flags': None,
                            'fs_type': 'fake-fs-type',
                            'type': 'fake-type',
                            'size': 1})]
@@ -117,18 +136,20 @@ class DiskPartitionerTestCase(base.IronicLibTestCase):
             check_exit_code=[0, 1])
         self.assertEqual(2, mock_utils_exc.call_count)
 
-    @mock.patch.object(eventlet.greenthread, 'sleep', lambda seconds: None)
     @mock.patch.object(disk_partitioner.DiskPartitioner, '_exec',
                        autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
     def test_commit_with_device_is_always_busy(self, mock_utils_exc,
                                                mock_disk_partitioner_exec):
+        CONF.set_override('check_device_interval', 0, group='disk_partitioner')
         dp = disk_partitioner.DiskPartitioner('/dev/fake')
         fake_parts = [(1, {'boot_flag': None,
+                           'extra_flags': None,
                            'fs_type': 'fake-fs-type',
                            'type': 'fake-type',
                            'size': 1}),
                       (2, {'boot_flag': 'boot',
+                           'extra_flags': None,
                            'fs_type': 'fake-fs-type',
                            'type': 'fake-type',
                            'size': 1})]
@@ -150,19 +171,20 @@ class DiskPartitionerTestCase(base.IronicLibTestCase):
             check_exit_code=[0, 1])
         self.assertEqual(20, mock_utils_exc.call_count)
 
-    # Mock the eventlet.greenthread.sleep for the looping_call
-    @mock.patch.object(eventlet.greenthread, 'sleep', lambda seconds: None)
     @mock.patch.object(disk_partitioner.DiskPartitioner, '_exec',
                        autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
     def test_commit_with_device_disconnected(self, mock_utils_exc,
                                              mock_disk_partitioner_exec):
+        CONF.set_override('check_device_interval', 0, group='disk_partitioner')
         dp = disk_partitioner.DiskPartitioner('/dev/fake')
         fake_parts = [(1, {'boot_flag': None,
+                           'extra_flags': None,
                            'fs_type': 'fake-fs-type',
                            'type': 'fake-type',
                            'size': 1}),
                       (2, {'boot_flag': 'boot',
+                           'extra_flags': None,
                            'fs_type': 'fake-fs-type',
                            'type': 'fake-type',
                            'size': 1})]
